/***********************************************************
 * $Id: FieldAccessor.java 4 2006-12-26 19:16:57Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.sec;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import sun.misc.BASE64Encoder;

/**
 * The SSHA1 hasher
 */
public class SSHA1PasswordHasher extends PlainPasswordHasher implements
		PasswordHasher
{
    // sun.misc.BASE64Encoder is not an officially documented JAVA
    // API, but it is at least implemented by SUN and IBM, thus making it
    // the most widely available base64 engine available in the wild.
    private BASE64Encoder base64encoder;
    
	public SSHA1PasswordHasher()
	{
		super();
		this.base64encoder = new BASE64Encoder();
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#hashPassword(java.lang.String, java.lang.String)
	 */
	@Override
    public String hashPassword(String cleartext, String salt)
	{
		return this.algo_prefix + sha1(cleartext, salt) + salt;
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#checkPassword(java.lang.String, java.lang.String)
	 */
	@Override
    public boolean checkPassword(String cleartext, String hashed)
	{
		String hash_salt = stripAlgorithmKey(hashed);
		int l = hash_salt.length(); // the naked hash is 20 bytes long, but
									// base64 makes it longer!
		String salt = hash_salt.substring(l - 4);
		String rehashed = hashPassword(cleartext, salt);
		return rehashed.equals(hashed);
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#getAlgorithmName()
	 */
	@Override
    public String getAlgorithmName()
	{
		return "SSHA1";
	}

	private String sha1(String cleartext, String salt)
	{
		try
		{
			// convert input to utf-8 based byte[]
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			OutputStreamWriter osw = new OutputStreamWriter(baos, "UTF-8");
			osw.write(cleartext);
			osw.write(salt);
			osw.close();
			baos.close();

			// make sha1 stuff
			MessageDigest sha1digest = MessageDigest.getInstance("SHA-1");
			sha1digest.update(baos.toByteArray());
			byte[] sha1ba = sha1digest.digest();
			String s = this.base64encoder.encode(sha1ba);

			return s + salt;
		} catch (NoSuchAlgorithmException e)
		{
			throw new RuntimeException(e);
		} catch (UnsupportedEncodingException e)
		{
			throw new RuntimeException(e);
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		}
	}
    
    
    /* (non-Javadoc)
     * @see org.clazzes.util.sec.PasswordHasher#getDefaultSaltLength()
     */
    public int getSaltLength() {
        return 4;
    }


}
