/***********************************************************
 * $Id: ClasspathLSResourceResolver.java 80 2007-12-08 11:08:44Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 07.12.2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.xml;

import java.util.HashMap;
import java.util.Map;

import javax.xml.XMLConstants;

import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSResourceResolver;

/**
 * This class resolves classpath-resident schema descriptions.
 * 
 * @author wglas
 */
public class ClasspathLSResourceResolver implements LSResourceResolver {

    private Map<String,String> xsdResources;
    
    /**
     * Construct a classpath resource resoulver.
     */
    public ClasspathLSResourceResolver() {
        this.xsdResources = new HashMap<String, String>();
    }
    
    /**
     * Add a classpath-resident schema description to this resource resolver.
     * 
     * @param nsURI
     * @param resourcePath
     */
    public void addXSDResource(String nsURI, String resourcePath) {
    
        this.xsdResources.put(nsURI,resourcePath);
    }
    
    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSResourceResolver#resolveResource(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public LSInput resolveResource(String type, String namespaceURI,
            String publicId, String systemId, String baseURI) {

        if (XMLConstants.W3C_XML_SCHEMA_NS_URI.equals(type))
        {
            String resourcePath = this.xsdResources.get(namespaceURI);
            
            if (resourcePath != null)
                return new ClasspathLSInput(resourcePath,publicId,systemId,baseURI);
        }
            
        return null;
    }

}
