/***********************************************************
 * $Id: SAXTagHandler.java 81 2007-12-08 20:40:37Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 08.12.2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.xml;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * An interface, which allows to implement SAX parsers as a stack of
 * tag handlers.
 * 
 * @author wglas
 */
public interface SAXTagHandler {
    
    /**
     * This function is called, when character data inside the processed
     * tag is received through {@link ContentHandler#characters(char[], int, int)}.
     * 
     * @param ch The character array.
     * @param start The start index inside the character array of the received data.
     * @param length The length of the received data.
     * @throws SAXException Upon errors processing the data.
     */
    public void processCharacters(char[] ch, int start, int length)
                                  throws SAXException;
    
    /**
     * This function is called, when a child element is opened inside the processed
     * tag through {@link ContentHandler#startElement(String, String, String, Attributes)}.
     * 
     * @param uri The namespace URI of the received tag. 
     * @param localName The name of the tag without any namespace prefix.
     * @param name The fully-qualified name of the tag. This is the tag name,
     *             if the namespace URI is null. 
     * @param atts Any attributes attached to the child 
     * @return The handler, which processes the child. A null return value
     *         tells the SAX parser to ignore the content of the child.
     * @throws SAXException Upon errors processing the data.
     */
    public SAXTagHandler startChildElement(String uri, String localName, String name,
                                           Attributes atts) throws SAXException;
    
    /**
     * This function is called, when the tag is closed.
     * 
     * @param uri The namespace URI of the tag. 
     * @param localName The name of the tag without any namespace prefix.
     * @param name The fully-qualified name of the tag. This is the tag name,
     *             if the namespace URI is null. 
     * @throws SAXException Upon errors processing the data.
     */
    public void closeTag(String uri, String localName, String name) throws SAXException; 
}
